# -------------------------------------------------------------
# Beispiel 9.8
# Freigabe mit Berechtigungen per WMI anlegen - lokal als auch im lokalen Netzwerk
# -------------------------------------------------------------
# Verschiedene Konstanten
$SHARE_READ = 1179817
$SHARE_CHANGE = 1245462
$SHARE_FULL = 2032127
$SHARE_NONE = 1

$ACETYPE_ACCESS_ALLOWED = 0
$ACETYPE_ACCESS_DENIED = 1

$ACEFLAG_INHERIT_ACE = 2

# -------------------------------------------------------------
# Legt ein neue Access Control Entry-Objekt (ACE) an
# -------------------------------------------------------------
function New-ACE
(
  [string]$Name, 
  [string]$Domain,
  [string]$Permission,
  [string]$ComputerName=".",
  [switch]$Group
)
{
    # Frischen Trustee (repräsentiert z.B. Benutzerkonto oder Gruppe) anlegen
    $Trustee = ([WMIClass]"\\$ComputerName\root\cimv2:Win32_Trustee").CreateInstance()
    # Soll eine Gruppe angelegt werden?
    if (!$Group)
    {
      # User lokalisieren
      $NTAccount = [WMI]"\\$ComputerName\root\cimv2:Win32_Account.Name='$Name',Domain='$Domain'"
    }
    else
    {
      # Gruppe lokalisieren
      $NTAccount = [WMI]"\\$ComputerName\root\cimv2:Win32_Group.Name='$Name',Domain='$Domain'"
    }
    # SID zum Benutzerkonto holen
    $NTAccountSID = [WMI]"\\$ComputerName\root\cimv2:Win32_SID.SID='$($NTAccount.SID)'"
    $Trustee.Domain = $Domain
    $Trustee.Name = $Name
    $Trustee.SID = $NTAccountSID.BinaryRepresentation
    # Frischen ACE anlagen
    $ACE = ([WMIClass]"\\$ComputerName\root\cimv2:Win32_ACE").CreateInstance()
    # Zugriffsmaske auswählen
    switch ($Permission)
    {
       "Full"   { $ACE.AccessMask = $SHARE_FULL }
       "Read"   { $ACE.AccessMask = $SHARE_READ }
       "Change" { $ACE.AccessMask = $SHARE_CHANGE }
       default { throw "Unbekannte Permission - erlaubt sind: 'Full', 'Read' und 'Change'"}
    }
    # ACE zusammensetzen
    $ACE.ACEType = 0
    $ACE.ACEFlags = $ACEFLAG_INHERIT_ACE
    $ACE.Trustee = $Trustee
    return $ACE
}

# -------------------------------------------------------------
# Legt einen neuen Security Descriptor an
# -------------------------------------------------------------
function New-SecurityDescriptor
( $ACEList, 
  $ComputerName ="."
)
{
  # Frischen Security Descriptor anlagen
  $SD = ([WMIClass]"\\$Computername\root\cimv2:Win32_SecurityDescriptor").CreateInstance()
  # Wurden ein oder mehrere ACEs übergeben?
  if ($ACEList.Length -ne $null)
  {
     # Alle ACEs durchgehen
     foreach($ACE in $ACEList)
     {
       $SD.DACL += $ACE.psObject.baseobject
     }
  }
  else
  {
      $SD.DACL = $ACEList
  }
  return $SD
}

# -------------------------------------------------------------
# Legt eine neue Freigabe an
# -------------------------------------------------------------
function New-Share
(
  $Path, $Name, $Description, $MaximumAllowed, $ACEList, $ComputerName="."
)
{
  if ($ACEList -ne $null)
  { $SD = New-SecurityDescriptor -ACEList $ACEList }
  # Frisches Win32_Share-Objekt anlagen
  $NewShare = [WMIClass]"\\$ComputerName\root\cimv2:Win32_Share"
  # Aufruf der Create-Methode
# Alternative 1: Traditionell über Create
  #$Ret = $NewShare.Create($Path, $Name, 0, $MaximumAllowed, $Description, $null, $SD)
  # Argumente für Create werden in alphabetischer Reihenfolge übergeben
  # Access, Description, MaximumAllowed, Name, Password, Path, Type
 # Alternative 2: Über Invoke-WmiMethod – wichtig Type Casting erforderlich!
  $Ret = Invoke-WmiMethod -Class Win32_Share -Name Create -ArgumentList @([System.Management.ManagementObject]$SD, $Description, $MaximumAllowed, $Name, $null, $Path, 0)

  return $Ret.ReturnValue
}

# Hier werden die Eckdaten der Freigabe zusammengestellt
$ACE1 = New-ACE -Name Pemo08 -Domain Haribo08 -Permission Read
$ACE2 = New-ACE -Name Administratoren -Domain Haribo08 -Permission Full –Group

$Ret = New-Share -Path "C:\PsKurs" -Name "PsKursNeu" -Description "Created by PowerShell" -MaximumAllowed 10 -ACEList $ACE1, $ACE2

switch ($Ret)
{
    0   {"Freigabe wurde angelegt."}
    2   { "Zugriff verweigert." }
    8   { "Unbekannter Fehler." }
    9   { "Ungültiger Name." }
    21  { "Ungültiger Parameter." }
    22 { "Freigabe existiert bereits." }
    default { "Unbekannter Rückgabewert." }
}
