# -------------------------------------------------------------
# Beispiel 9.15
# WMI-Events mit der PowerShell auswerten
# Hinzufgen eines USB-Laufwerks berwachen
# -------------------------------------------------------------

#requires -version 2.0

# Prft, ob der User Mitglied der Administratoren-Gruppe ist
function Is-Admin
{
    $CurUser = [System.Security.Principal.WindowsIdentity]::GetCurrent()
    (New-Object -Type System.Security.Principal.WindowsPrincipal `
      $CurUser).IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)
}

# Prft, ob das Skript unter XP ausfhren soll
function Is-WindowsXP
{
    !((Get-WmiObject -Class Win32_OperatingSystem).Version -gt "5.1.2600")
}

# Gibt zu jedem Event-Typ eine passende Meldung aus    
function Show-DriveEventType
($EventType)
{
    switch ($EventType)
    {
     1 { "Event-Typ: Konfigurationsnderung" }
     2 { "Event-Typ: Laufwerk hinzugefgt " }
     3 { "Event-Typ: Laufwerk entfernt " }
     4 { "Event-Typ: Docking-Aktion " }
     10 { "Event-Typ: besitzt Dateien- trotzdem Unbekannte Aktion " }
     default { "Unbekannter Event-Typ: $EventType" }
    }
}

# Wartet auf ein VolumeChangeEvent
function Get-DriveEvent
{
    Write-Host -Fore green -Back Black "Warten auf VolumeChangeEvent..."
    $NewEv = Wait-Event -SourceIdentifier USBWatch
    $NewDrive = $NewEv.SourceEventArgs.NewEvent.DriveName
    Write-Host -Fore Green -Back Black "nderung bei Laufwerk $NewDrive"
    Show-DriveEventType -EventType $NewEv.EventIdentifier
    # Wurde das Laufwerk hinzugefgt?
    if ($NewEv.EventIdentifier -eq 4)
    {
     # Kurz warten
     Start-Sleep 3
     Check-DriveContent -Drive $NewDrive
    }
    # Event aus der Warteschlange entfernen
    Remove-Event -SourceIdentifier USBWatch
}

# Prft vor der Formatierung, ob das Laufwerk einen Inhalt besitzt
function Check-DriveContent
($Drive)
{
    if ((Get-Childitem -Path $Drive -Ea SilentlyContinue).Count -gt 0)
    {
        $Ret = Read-Host -Prompt "Laufwerk besitzt Dateien- trotzdem formatieren? (J/N)"
        if ($Ret -like "N*")
        { 
          Write-Host -fore Green -Back Black "Laufwerk wird nicht formatiert - Skript wird beendet." 
        }
        else
        {
          Format-Drive -Drive $Drive
        }
    }
    else
    {
        $Ret = Read-Host -Prompt "Laufwerk formatieren? (J/N)"
        if ($Ret -like "J*")
        {
          Format-Drive -Drive $Drive
        }
    }
}

# Formatiert das Laufwerk
function Format-Drive
($Drive)
{
    Write-Host -Fore Magenta "NTFS-Formatierung von Laufwerk $Drive beginnt - das kann einige Minuten dauern..."
    $WmiDrive = Get-WmiObject -Class Win32_Volume -Filter "DriveLetter='$Drive'"
    $Ret = $WmiDrive.Format("NTFS", $True, 65536, "Formatiert by PowerShell", $False)
    switch ($Ret.ReturnValue)
    {
       0 { "Formatierung erfolgreich" }
       6 { "Formatierung nicht mglich - Volume schreibgeschtzt"  }
      15 { "Formatierung nicht mglich - Cluster-Gre zu klein"  }
      13 { "Formatierung nicht mglich - Volume zu gross"  }
      18 { "Formatierung nicht mglich - Unbekannter Fehler"      }
      default { "Formatierung nicht mglich - spezieller Fehler (Fehlercode: $($Ret.ReturnValue))" }
    }
 }

# Hier beginnt das Skript
try
{
    Unregister-Event USBWatch -ErrorAction Stop
}
catch {}
# Event eventuell entfernen
Remove-Event -SourceIdentifier USBWatch -ErrorAction SilentlyContinue
# Event registrieren
Register-WmiEvent -Class Win32_VolumeChangeEvent -SourceIdentifier USBWatch
if (!(Is-Admin))
{ Write-Host -fore red "Skript erfordert Administratorberechtigungen"; exit }
if (Is-WindowsXP)
{ Write-Host -fore red "Skript erfordert mind. Windows Server 2003"; exit }
# Auf Event warten
Get-DriveEvent
